/*!
 * \file main.c
 * \brief Renderiza&ccedil;&atilde;o dos caracteres ASCII em LCD
 * 
 * \details Este c&oacute;digo ilustra a inicializa&ccedil;&atilde;o e a transfer&ecirc;ncia de dados entre 
 *          um LCD e o MKZ25Z128 atrav&eacute;s de uma interface paralela.          
 * \mainpage Projeto de exibi&ccedil;&atilde;o de nome de cor selecionada no LCD.
 * \author Wu Shin-Ting
 * \date 01/09/2016
 * \attention O tempo de processamento do LCD &eacute; muito maior do que o do microcontrolador.
 *            &Eacute; preciso que os dois periféricos operem no mesmo passo quando eles 
 *	      est&atilce;o envolvidos numa mesma instru&ccedil;&atilde;o. Observe ainda 
 * 	      diferentes formas para armazenar uma string (sequ&ecirc;ncia de caracteres)
 * 	      na mem&oacute;ria. Lembrar que há sempre um byte de terminador adicional ('\0').
 *  
 * \note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
 *
 */

/*!
 * \fn main(void)
 * \attention Diferentes formas de declara&ccedil;&atilde;o e atribui&ccedil;&atilde;o 
 * s&atilde;o utilizadas. O importante &eacute; alocar um espa&ccedil;o de mem&oacute;ria
 * suficiente para armazenar a quantidade de caracteres (um byte por caractere).  
 *       
 */

#include "derivative.h" 
#include "stdlib.h"
#include "string.h"

#include "atrasos.h"
#include "ledRGB.h"
#include "pushbutton.h"
#include "lcdled.h"

 
/*!
 * Arranjo de caracteres de tamanho implicitamente definido pela inicializa&ccedil;&atilde;o "VERMELHO".
 */
char vermelho[] = "VERMELHO";
/*!
 * Ponteiro a um endere&ccedil;o de mem&oacute;ria inicializado com "BRANCO".
 */
char *branco = "BRANCO";
/*!
 * Arranjo de caracteres de tamanho explicitamente declarado e conte&uacute;do inicializado com "AMARELO"
 */
char amarelo[8] = {'A', 'M', 'A', 'R', 'E', 'L', 'O', '\0'};
/*!
 * Arranjo de caracteres de tamanho explicitamente declarado sem inicializa&ccedil;&atilde;o  
 */
char ciano[6]; 
/*!
 * Ponteiro a um endere&ccedil;o do tipo caractere
 */
char *magenta;
/*!
 * Ponteiro a um endere&ccedil;o do tipo caractere
 */
char *preto = "PRETO";

/*
 * \brief Mostra a cor dos leds no LCD
 * \param[in] ind &indice 
 */
void mostra_cor_LCD(uint8_t ind)
{

    /*! 
     * Mostre a cor no LCD
     */
    switch (ind) {
    case 0:
        mandaString(vermelho);
        break;
    case 1:
        mandaString(branco);
        break;
    case 2:
        mandaString(amarelo);
        break;
    case 3:
        mandaString(ciano);
        break;
    case 4:
        mandaString(magenta);
        break;
    case 5:
        mandaString(preto);
        break;            	
    }
}

/*!
* \brief Executa a fun&ccedil;&atilde;o de inicializa&ccedil;&atilde;o e ap&oacute;s se mant&eacute;m em um la&ccedil;o no qual &eacute; 
*     verificado o bot&atilde;o e em seguida controla a cor dos leds piscantes conforme o intervalo ds tempo que o bot&atilde;o ficou apertado.
* \return 1
*/
int main(void){
	uint8_t mudou;
    short pta;
    uint8_t indice5, indice12;
    char limpa[] = "                ";
    cor cores_estaticas[] = {{ACESO,APAGADO,APAGADO},{ACESO,ACESO,ACESO},
    						 {ACESO,ACESO,APAGADO},{APAGADO,ACESO,ACESO},
    						 {ACESO,APAGADO,ACESO},{APAGADO,APAGADO,APAGADO}};
    float freqs[] = {0.5,1.0,5.0,15.0,30.0,60.0};
    float *tmp;
    
    uint8_t icon_sino[8] = {0x04, 0x0e, 0x0e, 0x0e, 0x1f, 0x00, 0x04, 0x00} ;
    
    /*!
     * Inicializa&ccedil;&atilde;o das strings
     */
     ciano[0] = 'C'; ciano[1]='I'; memcpy(&ciano[2], "ANO", sizeof(char)*4);
	
     magenta = malloc(sizeof(char)*8);    ///< aloca 8 bytes para "MAGENTA"
     magenta = "MAGENTA";

     preto = malloc(sizeof(char)*6);      ///< aloca 6 bytes para "PRETO"
     preto[0] = 'P'; preto[1] = 'R'; preto[2] = 'E'; preto[3] = 'T'; preto[4] = 'O'; preto[5]= '\0'; 
    
   /*!
     * Inicializa&ccedil;&otilde;es do ledRGB
     */
    initLedRGB();
    
    /*!
     * Inicializa&ccedil;&otilde;es dos push buttons
     */
    initPushbuttonIRQA ();

    /*!
     * Inicializa&ccedil;&otilde;es do LCD
     */
    initLcdledGPIO();
    initLCD();
    
    /*!
     * Cria um novo bitmap no endere&ccedil;o 0
     */
    criaBitmap(xxxx,icon_sino);
    
    /*!
     * Inicializa&ccedil;&atilde; das vari&aacute;veis
     */
    mudou = 1;               ///< estado da botoeira
    indice5 = indice12 = 5;  ///< cor e frequ&ecirc;ncia iniciais
    
    /*! 
     * Execu&ccedil;&atilde;o da tarefa pisca-pisca sob controle
     */
    for(;;){
            pta = le_pta(PTA5);
            /*!
             * Se bot&atilde;o de controle de frequ&ecirc;ncia &eacute; apertado
             */
            if (!pta)
            {
              enviaLed(0xFF);               ///< liga leds vermelhos 
              /*! ent&atilde;o estima o intervalo de tempo que ficou neste estado */
              indice5 = tempo_espera(PTA5);
              enviaLed(0x00);               ///< apaga leds vermelhos
              mudou = 1;
            }

            pta = le_pta(PTA12);
            /*!
             * Se bot&atilde;o de controle de cor &eacute; apertado
             */
            if (!pta)
            {
              enviaLed(0xFF);               ///< liga leds vermelhos 
              /*! ent&atilde;o estima o intervalo de tempo que ficou neste estado */
              indice12 = tempo_espera(PTA12);
              enviaLed(0x00);               ///< apaga leds vermelhos
              mudou = 1;
            }
            
            if (mudou) {
            	/*! Mostra a cor no LCD */
            	RS(0);
            	enviaLCD(0b11000000, xxxx);     ///< posicione o cursor
            	RS(1);
            	mandaString(limpa);    ///< limpe os campos
            
            	RS(0);
            	enviaLCD(0b11000000, xxxx);       ///< reposicione o cursor
            	RS(1);
            	enviaLCD(0x00,xxxx);      ///< envia o caractere customizado 
            	mostra_cor_LCD(indice12); ///< escreva o nome da cor
            	mudou = 0;
            }
            /*!
             * Exibe a cor conforme o intervalo de tempo estimado
             */
        	led (VERMELHO, *((estado_led *)(cores_estaticas[indice12].vermelho)));
        	led (VERDE, *((estado_led *)(cores_estaticas[indice12].verde)));
        	led (AZUL, *((estado_led *)(cores_estaticas[indice12].azul)));

        	/*!
        	 * Pisca conforme o intervalo de tempo estimado
        	 */
        	tmp = &freqs[indice5];
        	delayfreq(*tmp);                /*! Espera 1/(2*freqs[indice5]) */
        	
            /*!
             * Se o estado dos leds n&atilde;o for apagado (indice12 != 5)
             */
            if (indice12 != 5)
            {
                /*! Apaga os leds  */
                led (VERMELHO, APAGADO);
                led (VERDE, APAGADO);
                led (AZUL, APAGADO);
                
                delayfreq(*tmp);           /*! Espera 1/(2*freqs[indice5]) */
            }

    }

    return 1;
}            
